package tsp.csc4509.spool;

import static tsp.csc4509.common.Log.GEN;

import java.io.FileInputStream;
import java.io.Serializable;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Scanner;

public class Information implements Serializable{
	private static final long serialVersionUID = 1L;
	public static final String INFORMATIONFILENAME = "information.txt" ;
	public static final String FROM = "From:";
	public static final String TO = "To:";
	public static final String FILENAME = "Filename:";
	private static final String FORMATERROR = "Le fichier " + INFORMATIONFILENAME + " est mal formaté.";
	
	/**
	 * nom de l'utilisateur expéditeur du fichier. 
	 */
	private String from;
	/**
	 * nom de l'utilisateur destinataire du fichier.
	 */
	private String to;
	/**
	 * nom du fichier sans la partie répertoire.
	 */
	private String fileName;
	

	/**
	 * Constructeur qui initialise les trois champs des attributs à partir de leur valeur. 
	 * @param from
	 *          nom de l'utilisateur expéditeur du fichier. 
	 * @param to
	 *          nom de l'utilisateur destinataire du fichier. Peut être null pour
	 *          la requête GET (pas de fichier émis).
	 * @param fileName
	 *          nom du fichier (complet, avec le répertoire) à émettre. Peut être null pour
	 *          la requête GET (pas de fichier émis).
	 */
	public Information(final String from, final String to, final String fileName) {
		super();
		this.from = from;
		this.to = to;
		if (fileName != null) {
		// on coupe le partie répertoire du fileName pour ne garder que le nom du fichier.
			this.fileName = Paths.get(fileName).getFileName().toString();
		} else {
			this.fileName = null;
		}
	}

	/**
	 * Constructeur  qui initialise les trois champs des attributs à partir du 
	 * fichier information contenu dans le répertoire de Spool dir.
	 * @param dir
	 *          chemin d'un répertoire du Spool contenant un fichier.
	 * @throws Exception
	 *          toutes les exceptions levées lors de la consultation du fichier
	 *          d'information.
	 */
	public Information(final Path dir) throws Exception {
		Path infoFilePath = dir.resolve(Information.INFORMATIONFILENAME);
		try (FileInputStream infoStream = new FileInputStream(infoFilePath.toString());
				Scanner infoScanner = new Scanner(infoStream); 
				) {
			String fromLine = infoScanner.nextLine();
			GEN.info("FromLine: " + fromLine);
			if (!fromLine.startsWith(FROM)) {
				throw new Exception(FORMATERROR);
			}
			from = fromLine.split(":")[1];
			if (from.length() < 1) {
				throw new Exception(FORMATERROR);
			}
			
			String toLine = infoScanner.nextLine();
			GEN.info("toLine: " + toLine);
			if (!toLine.startsWith(TO)) {
				throw new Exception(FORMATERROR);
			}
			to = toLine.split(":")[1];
			if (to.length() < 1) {
				throw new Exception(FORMATERROR);
			}
			String fileLine = infoScanner.nextLine();
			GEN.info("fileLine: " + fileLine);
			if (!fileLine.startsWith(FILENAME)) {
				throw new Exception(FORMATERROR);
			}
			fileName = fileLine.split(":")[1];
			if (fileName.length() < 1) {
				throw new Exception(FORMATERROR);
			}
			GEN.info("Information construit");
		} 
		
	}
	
	public String getFrom() {
		return from;
	}

	public void setFrom(String from) {
		this.from = from;
	}

	public String getTo() {
		return to;
	}

	public void setTo(String to) {
		this.to = to;
	}

	public String getFileName() {
		return fileName;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	@Override
	public String toString() {
		return "Information [from=" + from + ", to=" + to + ", fileName=" + fileName + "]";
	}
	



}
