package tsp.csc4509.tpnote;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.SocketChannel;

import tsp.csc4509.common.Log;

/**
 * Interface fournissant deux méthodes par défaut pour l'envoi et la réception
 * en mode synchrone d'un fichier à travers le réseau.
 * 
 * @author
 *
 */
public interface SenderReceiver {
	/**
	 * Taille des ByteBuffer utilisés pour la réception et l'émission des fichier.
	 */
	final int BUFFSIZE = 1024;

	// Q1
	/**
	 * Reception en mode synchrone d'un fichier via le réseau.
	 * 
	 * @param fcout  Canal déjà relié à un fichier créé et ouvert en écriture.
	 * @param rwChan Canal déjà ouvert sur une connexion Tcp.
	 * @throws IOException Toutes les exceptions d'entrées/sorties.
	 */
	default void recvFile(final FileChannel fcout, final SocketChannel rwChan) throws IOException {
		/* #################### Debut corrigé Question 1 ##################### */
		ByteBuffer buffer = ByteBuffer.allocate(BUFFSIZE);
		int nb;
		do {
			buffer.clear();
			nb = rwChan.read(buffer);
			//Log.GEN.info("reçu {}", nb);
			if (nb == -1) {
				return;
			}
			buffer.flip();
			fcout.write(buffer);
		} while (nb > 0);
		/* #################### Fin corrigé Question 1 ##################### */
	}

	// Q2
	/**
	 * Envoi d'un fichier via le réseau.
	 * 
	 * @param fcin   Canal déjà relié à un fichier ouvert en lecture.
	 * @param rwChan Canal déjà ouvert sur une connexion Tcp.
	 * @throws IOException Toutes les exceptions d'entrées/sorties.
	 */
	default void sendFile(final FileChannel fcin, final SocketChannel rwChan) throws IOException {
		/* #################### Debut corrigé Question 2 ##################### */
		ByteBuffer buffer = ByteBuffer.allocate(BUFFSIZE);
		int nb;
		do {
			buffer.clear();
			nb = fcin.read(buffer);
		//	Log.GEN.info("Lu {}", nb);
			if (nb == -1) {
				return;
			}
			buffer.flip();
			rwChan.write(buffer);
		} while (nb == BUFFSIZE);
		/* #################### Fin corrigé Question 2 ##################### */
	}
}
