package tsp.csc4509.tpnote;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.nio.channels.FileChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.file.Path;
import java.util.concurrent.Callable;

import tsp.csc4509.spool.Spool;

/**
 * Callable pour le thread qui prend en charge l'envoi et la réception des fichiers
 * en mode synchrone coté serveur.
 * Le call() retourne le chemin vers le répertoire contenant les informations sur
 * le fichier reçu ou émis.
 *  
 * @author 
 *
 */
public class ServerDataTransfertProcessCallable implements Callable<Path>, SenderReceiver {
	/**
	 * Socket serveur crée établir la connexion pour échnager les données contenues
	 * dans le fichier à recevoir ou à émettre. 
	 */
	private ServerSocketChannel listenChannel;
	/**
	 * Chemin vers le répertoire contenant les informations sur le fichier à recevoir ou à émettre.
	 */
	private Path path;
	/**
	 * Message du protocole qui a demandé la réception (message PUT) ou l'envoi (message GET) d'un fichier.
	 */
	private final ProtocolMessage message;


	/**
	 * Constructeur pour initiliser les trois attributs du Callable. 
	 * @param listenChannel
	 *             ServerSocketChannel du DTP du serveur crée pour établir la connexion pour échanger les données contenues
	 *             dans le fichier à recevoir ou à émettre. 
	 * @param path
	 *             Chemin vers le répertoire contenant les informations sur le fichier à recevoir ou à émettre.
	 * @param message
	 *              Message du protocole qui a demandé la réception (message PUT) ou l'envoi (message GET) d'un fichier.
	 */
	public ServerDataTransfertProcessCallable(final ServerSocketChannel listenChannel, final Path path, final ProtocolMessage message) {
		super();
		this.listenChannel = listenChannel;
		this.path = path;
		this.message = message;
	}

	@Override
	public Path call() throws Exception {
		/*#################### Debut corrigé Question 7 #####################*/
		try (SocketChannel clientChan = listenChannel.accept();) {
			switch (message.getMessageType()) {
			case PUT: {
				FileOutputStream fout = Spool.getOutputFile(path);
				FileChannel fcout = fout.getChannel();
				recvFile(fcout, clientChan);
				fcout.close();
				fout.close();
				break;
			}
			case GET: {
				FileInputStream fin = Spool.getInputFile(path);
				FileChannel fcin = fin.getChannel();
				sendFile(fcin, clientChan);
				fcin.close();
				fin.close();
				break;
			}
			default:
				// rien à faire
			// les deux autres messages du protocole (SEND et RECEIVE) circulent dans le
			// sens serveur vers client et n'ont pas à être gérés ici.
			}
			return path;
		}
		/*#################### Fin corrigé Question 7 #####################*/
	}
}
