package tsp.csc4509.tpnote;

import static tsp.csc4509.common.Log.TEST;
import static tsp.csc4509.common.Log.COMM;
import org.apache.logging.log4j.Level;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.StandardSocketOptions;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.file.Path;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

import org.junit.jupiter.api.io.TempDir;

import tsp.csc4509.common.Log;
import tsp.csc4509.spool.Information;
import tsp.csc4509.spool.Spool;
import tsp.csc4509.tcpnio.FullDuplexMsgWorker;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.io.TempDir;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertAll;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class TestClientEricLallet {
	private static final String USERIDCLIENT = "userTest";
	private static final String USERIDFROM = "userFrom";
	private static final String TESTFILEFILENAME = "send.data"; 
	private static final String USERSPOOL = Client.CLIENTSSPOOLDIR;


	/**
	 * numéro du port utilisé par les serveur lancés pour les tests.
	 */
	private static final int TESTSERVERPORT = 40001; 

	/**
	 * premier numéro des ports utilisés pour le DTP.
	 */
	private static final int TESTSERVERDTPPORT = 20001; 

	/**
	 * thread pour mettre de alarm en place pour détecter les blocages..
	 */
	private static Thread alarmThread = null;
	private static boolean alarmOn = false;

	/**
	 * durée des attentes lorsqu'une alarme est lancée (en milliseconde).
	 */
	private static final int ALARMTIMEOUT = 4000;

	/**
	 * taille des ByteBuffer.
	 */
	private static final int BUFFERSIZE = 1024;
	/**
	 * Taille du fichier DATATESTFILE. Pour tester l'envoi et la réception
	 * en plusieurs sendBuffer()/receiveBuffer(), cette taille doit être
	 * plus grande que BUFFERSIZE.
	 */
	private static final int DATATESTFILESIZE = (int) (2.5 * BUFFERSIZE);

	private static Thread getThread ;

	@BeforeAll
	public static void initTest() {
//				Log.setLevel(TEST, Level.TRACE);
//				Log.setLevel(COMM, Level.TRACE);
	}


	//========================================================
	// test 1: Si le serveur se déconnecte imméfiatement sans envoyer aucun
	// REVEIVE, get() doit le détecter, et terminet immédiatement.
	@Disabled
	@Test
	public void testGet1(@TempDir Path tempDir) throws Exception {
		// 
		TEST.trace("(TestClient::testGet1]");

		FutureTask<Void> task = new FutureTask<>( new Callable<Void>() {

			@Override
			public Void call() throws Exception {
				try (
						ServerSocketChannel listenChan = createServerSocketChannel(TESTSERVERPORT);
						SocketChannel rwChan = listenChan.accept();
						)
				{
					// syntaxe try {} le serveur ferme tout dès qu'un accept est fait.
					return null;
				}
			}
		}
				);

		// lancement du thread qui contient le serveur.
		new Thread(task).start();
		Thread.sleep(1000);


		// création du client qui se connecte au serveur.
		Client client = new Client(USERIDCLIENT, "localhost", TESTSERVERPORT);


		// lancement d'un threag qui réalise le get() à tester.
		getThread = new Thread(new Runnable() {

			@Override
			public void run() {
				try {
					client.get();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}

		});
		getThread.start();


		// aucun message n'est envoyé au client, il doit se terminer immédatement.
		alarmStart(getThread, ALARMTIMEOUT);
		getThread.join();
		alarmStop(null);

		assertTrue(true, "Le get se termine quand il n'y a pas de requête et que le serveur a fermé la connexion"); 
	}



	//========================================================
	// test 2: le serveur reste connecté: le get() doit rester en attente d'une requête RECEIVE.
	@Disabled
	@Test
	public void testGet2(@TempDir Path tempDir) throws IOException, ExecutionException, InterruptedException {
		// 
		TEST.trace("(TestClient::testGet2]");


		// callable simulant le serveur.
		FutureTask<SocketChannel> taskPI = new FutureTask<>( new Callable<SocketChannel>() {
			@Override
			public SocketChannel call() throws Exception {
				try (ServerSocketChannel listenChan = createServerSocketChannel(TESTSERVERPORT);
						)
				{
					SocketChannel rwChan = listenChan.accept();
					//		Information info = new Information(USERIDFROM, USERIDCLIENT, TESTFILEFILENAME);

					// la connexion relié au client n'est pas fermée
					return rwChan;
				}
			}
		});

		// lancement du thread qui contient le serveur.
		new Thread(taskPI).start();
		Thread.sleep(1000);

		// création du client qui se connecte au serveur.
		Client client = new Client(USERIDCLIENT, "localhost", TESTSERVERPORT);

		// on reccupère le canal coté serveur.
		SocketChannel serverPIChan = taskPI.get();

		// lancement d'un thread qui réalise le get() à tester.
		getThread = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					client.get();
				} catch (Exception e) {
					TEST.trace("" + e);
					e.printStackTrace();
				}
			}
		});
		getThread.start();


		// la connexion n'est pas fermé. Le get doit attendre. 
		alarmStart(getThread, ALARMTIMEOUT);	
		getThread.join(); // le join ne doit pas se produire. Le Thread doit être interrompu
		
		assertTrue(alarmOn, "Le get() est bien resté en attente");

		alarmStop(null);

	}

	//========================================================
	// test 3: le serveur envoie requête RECEIVE. Le get() doit de connecter au DTP du serveur.
	@Disabled
	@Test
	public void testGet3(@TempDir Path tempDir) throws IOException, ExecutionException, InterruptedException {
		// 
		TEST.trace("(TestClient::testGet3]");
		
		// callable simulant le serveur qui envoie une requête RECEIVE
		FutureTask<SocketChannel> taskPI = new FutureTask<>( new Callable<SocketChannel>() {
			@Override
			public SocketChannel call() throws Exception {
				try (ServerSocketChannel listenChan = createServerSocketChannel(TESTSERVERPORT);
						)
				{
					SocketChannel rwChan = listenChan.accept();
					
					FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
					ServerSocketChannel dtpListenChan = createServerSocketChannel(TESTSERVERDTPPORT);
					Information info = new Information(USERIDFROM, USERIDCLIENT, TESTFILEFILENAME);
					ProtocolMessage receiveMessage = new ProtocolMessage(ProtocolMessageType.RECEIVE, info, TESTSERVERDTPPORT);
					worker.sendMsg(MessType.PROTOCOLMESSAGETYPE, receiveMessage);
					SocketChannel dtpRwChan = dtpListenChan.accept();
					

					worker.close();
					return dtpRwChan;
				}
			}
		});

	
		// lancement du thread qui contient le serveur.
		Thread threadPI = new Thread(taskPI);
		threadPI.start();
		
		Thread.sleep(1000);

		// création du client qui se connecte au serveur.
		Client client = new Client(USERIDCLIENT, "localhost", TESTSERVERPORT);

		
	


		// lancement d'un thread qui réalise le get() à tester.
		// doit se connecter au DTP du serveur.
		getThread = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					client.get();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		getThread.start();
		


		alarmStart(threadPI, ALARMTIMEOUT);
		SocketChannel dtpServerChan = taskPI.get();
		alarmStop(null);

		assertNotNull(dtpServerChan, "Le get s'est bien connecté au DTP");

	}



	//========================================================
	// test 4: test de le get() recoit les données envoyées sur son DTP
	@Disabled
	@Test
	public void testGet4(@TempDir Path tempDir) throws Exception {
		// 
		TEST.trace("(TestClient::testGet4]");
		final String SENDFILENAME = tempDir.resolve(TESTFILEFILENAME).toString();
		createDataTestFile(SENDFILENAME, 0, DATATESTFILESIZE);

		// callable simulant le serveur qui envoie une requête RECEIVE + 1 fichier
		FutureTask<SocketChannel> taskPI = new FutureTask<>( new Callable<SocketChannel>() {
			@Override
			public SocketChannel call() throws Exception {
				try (ServerSocketChannel listenChan = createServerSocketChannel(TESTSERVERPORT);
						)
				{
					SocketChannel rwChan = listenChan.accept();
					
					FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
					ServerSocketChannel dtpListenChan = createServerSocketChannel(TESTSERVERDTPPORT);
					Information info = new Information(USERIDFROM, USERIDCLIENT, TESTFILEFILENAME);
					ProtocolMessage receiveMessage = new ProtocolMessage(ProtocolMessageType.RECEIVE, info, TESTSERVERDTPPORT);
					worker.sendMsg(MessType.PROTOCOLMESSAGETYPE, receiveMessage);
					SocketChannel dtpRwChan = dtpListenChan.accept();
					FileInputStream fin = new FileInputStream(SENDFILENAME);
					FileChannel fcin = fin.getChannel();
					ByteBuffer buffer = ByteBuffer.allocate(DATATESTFILESIZE);
					fcin.read(buffer);
					fin.close();
					fcin.close();
					buffer.flip();
					dtpRwChan.write(buffer);
					dtpRwChan.close();

					worker.close();
					return dtpRwChan;
				}
			}
		});

	
		// lancement du thread qui contient le serveur.
		Thread threadPI = new Thread(taskPI);
		threadPI.start();
		
		Thread.sleep(1000);

		// création du client qui se connecte au serveur.
		Client client = new Client(USERIDCLIENT, "localhost", TESTSERVERPORT);

		
	


		// lancement d'un thread qui réalise le get() à tester.
		// doit se connecter au DTP du serveur.
		getThread = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					client.get();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		getThread.start();
		

		// le get doit se terminer après le réception du fichier/
		alarmStart(getThread, ALARMTIMEOUT);
		getThread.join(); 
		assertFalse(alarmOn, "Le get() s'est terminé avant le timeout");
		alarmStop(null);
		
		
		// Le test fait l'hypothèse que le spool de USERSPOOL a été réinitialisé avant le test et que le fichier sera dans le répertoire 1
		assertTrue(compareFile(SENDFILENAME, USERSPOOL+ "/" + USERIDCLIENT + "/1/file/" + TESTFILEFILENAME ), "Le fichier dans le spool client identique au fichier émis");
	}
	
	
	//========================================================
	// test 5: test de le get() recoit les données envoyées sur 2 DTP en même temps 
	@Disabled
	@Test
	public void testGet5(@TempDir Path tempDir) throws Exception {
		// 
		TEST.trace("(TestClient::testGet4]");
		final String SENDFILENAME = tempDir.resolve(TESTFILEFILENAME).toString();
		createDataTestFile(SENDFILENAME, 0, DATATESTFILESIZE);

		// callable simulant le serveur qui envoie une requête RECEIVE + 1 fichier
		FutureTask<SocketChannel> taskPI = new FutureTask<>( new Callable<SocketChannel>() {
			@Override
			public SocketChannel call() throws Exception {
				try (ServerSocketChannel listenChan = createServerSocketChannel(TESTSERVERPORT);
						)
				{
					SocketChannel rwChan = listenChan.accept();
					
					FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
					ServerSocketChannel dtpListenChan1 = createServerSocketChannel(TESTSERVERDTPPORT);
					ServerSocketChannel dtpListenChan2 = createServerSocketChannel(TESTSERVERDTPPORT+1);
					Information info = new Information(USERIDFROM, USERIDCLIENT, TESTFILEFILENAME);
					ProtocolMessage receiveMessage1 = new ProtocolMessage(ProtocolMessageType.RECEIVE, info, TESTSERVERDTPPORT);
					ProtocolMessage receiveMessage2 = new ProtocolMessage(ProtocolMessageType.RECEIVE, info, TESTSERVERDTPPORT+1);
					// 2 receiveMessage envoyés dans un ordre différent de celui des accept plus loin
					worker.sendMsg(MessType.PROTOCOLMESSAGETYPE, receiveMessage2);
					worker.sendMsg(MessType.PROTOCOLMESSAGETYPE, receiveMessage1);
					
					FileInputStream fin1 = new FileInputStream(SENDFILENAME);
					FileChannel fcin1 = fin1.getChannel();
					FileInputStream fin2 = new FileInputStream(SENDFILENAME);
					FileChannel fcin2 = fin2.getChannel();
					ByteBuffer buffer1 = ByteBuffer.allocate(BUFFERSIZE);
					ByteBuffer buffer2 = ByteBuffer.allocate(BUFFERSIZE);
					
					// premier DTP
					SocketChannel dtpRwChan1 = dtpListenChan1.accept();
					fcin1.read(buffer1);
					buffer1.flip();
					dtpRwChan1.write(buffer1); // début du fichier envoyé
					
					// second DTP
					SocketChannel dtpRwChan2 = dtpListenChan2.accept();
					fcin2.read(buffer2);
					buffer2.flip();
					dtpRwChan2.write(buffer2); // début du fichier envoyé
					
					int lu;
					do {
						buffer1.clear();
						lu = fcin1.read(buffer1);
						buffer1.flip();
						dtpRwChan1.write(buffer1);	
					} while (lu > 0);
					fin1.close();
					fcin1.close();
					dtpRwChan1.close();
					
					do {
						buffer2.clear();
						lu = fcin2.read(buffer2);
						buffer2.flip();
						dtpRwChan2.write(buffer2);	
					} while (lu > 0);
					fin2.close();
					fcin2.close();
					dtpRwChan2.close();
				

					worker.close();
					return dtpRwChan1;
				}
			}
		});

	
		// lancement du thread qui contient le serveur.
		Thread threadPI = new Thread(taskPI);
		threadPI.start();
		
		Thread.sleep(1000);

		// création du client qui se connecte au serveur.
		Client client = new Client(USERIDCLIENT, "localhost", TESTSERVERPORT);

		
	


		// lancement d'un thread qui réalise le get() à tester.
		// doit se connecter au DTP du serveur.
		getThread = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					client.get();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		getThread.start();
		

		// le get doit se terminer après le réception des fichiers.
		alarmStart(getThread, ALARMTIMEOUT);
		getThread.join(); 
		assertFalse(alarmOn, "Le get() s'est terminé avant le timeout");
		alarmStop(null);
		
		
		// Le test fait l'hypothèse que le spool de USERSPOOL a été réinitialisé avant le test et que le fichier sera dans le répertoire 1
		assertTrue(compareFile(SENDFILENAME, USERSPOOL+ "/" + USERIDCLIENT + "/1/file/" + TESTFILEFILENAME ), "Le fichier 1 dans le spool client identique au fichier émis");
		assertTrue(compareFile(SENDFILENAME, USERSPOOL+ "/" + USERIDCLIENT + "/2/file/" + TESTFILEFILENAME ), "Le fichier 2 dans le spool client identique au fichier émis");
		
		

	}
	
	//========================================================
	// test 6: test de le get() recoit 1 requête RECEIVE + le fichier +  1 requête RECEIVE + le fichier
//	@Disabled
	@Test
	public void testGet6(@TempDir final Path tempDir) throws Exception {
		// 
		TEST.trace("(TestClient::testGet6]");
		final String SENDFILENAME = tempDir.resolve(TESTFILEFILENAME).toString();
		createDataTestFile(SENDFILENAME, 0, DATATESTFILESIZE);

		// callable simulant le serveur qui envoie une requête RECEIVE + 1 fichier + une requête RECEIVE + 1 fichier 
		FutureTask<SocketChannel> taskPI = new FutureTask<>(new Callable<SocketChannel>() {
			@Override
			public SocketChannel call() throws Exception {
				try (ServerSocketChannel listenChan = createServerSocketChannel(TESTSERVERPORT);) {
					SocketChannel rwChan = listenChan.accept();

					FullDuplexMsgWorker worker = new FullDuplexMsgWorker(rwChan);
					ServerSocketChannel dtpListenChan1 = createServerSocketChannel(TESTSERVERDTPPORT);
					ServerSocketChannel dtpListenChan2 = createServerSocketChannel(TESTSERVERDTPPORT + 1);
					Information info = new Information(USERIDFROM, USERIDCLIENT, TESTFILEFILENAME);
					ProtocolMessage receiveMessage1 = new ProtocolMessage(ProtocolMessageType.RECEIVE, info,
							TESTSERVERDTPPORT);
					ProtocolMessage receiveMessage2 = new ProtocolMessage(ProtocolMessageType.RECEIVE, info,
							TESTSERVERDTPPORT + 1);
					// 2 receiveMessage envoyés dans un ordre différent de celui des accept plus loin

					worker.sendMsg(MessType.PROTOCOLMESSAGETYPE, receiveMessage1);

					FileInputStream fin1 = new FileInputStream(SENDFILENAME);
					FileChannel fcin1 = fin1.getChannel();
					FileInputStream fin2 = new FileInputStream(SENDFILENAME);
					FileChannel fcin2 = fin2.getChannel();
					ByteBuffer buffer1 = ByteBuffer.allocate(BUFFERSIZE);
					ByteBuffer buffer2 = ByteBuffer.allocate(BUFFERSIZE);

					// premier DTP
					SocketChannel dtpRwChan1 = dtpListenChan1.accept();	
					int lu;
					do {
						buffer1.clear();
						lu = fcin1.read(buffer1);
						buffer1.flip();
						dtpRwChan1.write(buffer1);
					} while (lu > 0);
					fin1.close();
					fcin1.close();
					dtpRwChan1.close();
					// la requête 1 entièrement envoyée.... mais tout n'est pas fini.

					Thread.sleep(1000);

					worker.sendMsg(MessType.PROTOCOLMESSAGETYPE, receiveMessage2);
					// second DTP
					SocketChannel dtpRwChan2 = dtpListenChan2.accept();
					do {
						buffer2.clear();
						lu = fcin2.read(buffer2);
						buffer2.flip();
						dtpRwChan2.write(buffer2);
					} while (lu > 0);
					fin2.close();
					fcin2.close();
					dtpRwChan2.close();

					// tout est fini
					worker.close();
					return dtpRwChan1;
				}
			}
		});

		// lancement du thread qui contient le serveur.
		Thread threadPI = new Thread(taskPI);
		threadPI.start();

		Thread.sleep(1000);

		// création du client qui se connecte au serveur.
		Client client = new Client(USERIDCLIENT, "localhost", TESTSERVERPORT);

		// lancement d'un thread qui réalise le get() à tester.
		// doit se connecter au DTP du serveur.
		getThread = new Thread(new Runnable() {
			@Override
			public void run() {
				try {
					client.get();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		getThread.start();

		// le get doit se terminer après le réception des fichiers.
		alarmStart(getThread, ALARMTIMEOUT);
		getThread.join();
		assertFalse(alarmOn, "Le get() s'est terminé avant le timeout");
		alarmStop(null);

		// Le test fait l'hypothèse que le spool de USERSPOOL a été réinitialisé avant le test et que le fichier sera dans le répertoire 1
		assertTrue(compareFile(SENDFILENAME, USERSPOOL + "/" + USERIDCLIENT + "/1/file/" + TESTFILEFILENAME),
				"Le fichier 1 dans le spool client identique au fichier émis");
		assertTrue(compareFile(SENDFILENAME, USERSPOOL + "/" + USERIDCLIENT + "/2/file/" + TESTFILEFILENAME),
				"Le fichier 2 dans le spool client identique au fichier émis");
	}

	



	/**
	 * Création d'un ServerSocketChannel lié au port passé en paramètre.
	 * @param port
	 *          numéro du port du ServerSocketChannel.
	 * @return
	 *          ServerSocketChannel lié au port.
	 * @throws IOException
	 *          toutes les exceptions pouvant se produire lors de la création
	 *          et du bind du ServerSocketChannel.
	 */
	public  ServerSocketChannel createServerSocketChannel(final int port) throws IOException {
		ServerSocketChannel listenChannel = ServerSocketChannel.open();
		listenChannel.setOption(StandardSocketOptions.SO_REUSEADDR, true);
		listenChannel.bind(new InetSocketAddress(port));
		return listenChannel;
	}

	/**
	 * Débute une alarme qui enverra une intérruption au thread t dans timeout milisecondes.
	 * Utiliser alarmStop() pour arrêter cette alarme.
	 * @param timeout
	 *           durée en millisecondes avant d'envoyer l'interruption.
	 */
	public static void alarmStart(final Thread t, long timeout) {
		Runnable alarmRun = new Runnable() {
			@Override
			public void run() {
				try {

					//	TEST.trace("{}", () ->  "TcpTestTools: Lancement du service d'alarme pour une durée de " + timeout);

					alarmOn = false;
					Thread.sleep(timeout);
					t.interrupt();
					alarmOn = true;
					t.stop();

				} catch (Exception e1) {
					if (e1.getMessage() != null) {

					}
					// fin du thread sans autre explication... TODO traiter les fins anormales!
				} 
				alarmThread = null;

			}

		};
		alarmThread = new Thread(alarmRun);
		alarmThread.start();
	}

	/**
	 * Interrompt un Thread lancé précédemment. 
	 * @param message
	 *         message de service qui annonce la fin du thread dans le log niveau trace.
	 *         Passer une chaîne null pour rendre la méthode muette.
	 * @throws InterruptedException
	 *         exception en cas d'interruption pendant join().
	 */
	private static void alarmStop(final String message) throws InterruptedException {

		if (message != null) {
			TEST.trace("TcpTestTools: " +  message);
		}
		alarmOn = false;

		if ((alarmThread != null) && (alarmThread.isAlive())) {
			alarmThread.interrupt();
			alarmThread.join();
		}


		if (message != null) {
			TEST.trace("alarmThread: Thread terminé");
		}
		alarmThread = null;


	}

	/**
	 * Compare le contenu de deux fichiers et retourne true s'ils sont identiques.
	 * @param fileName1
	 * 					nom du premier fichier.
	 * @param fileName2
	 * 					nom du second fichier.
	 * @return
	 * 					true si les deux fichiers ont le même contenu, false sinon.
	 * @throws Exception
	 * 					toutes les exceptions possibles.
	 */
	private static  boolean compareFile(final String fileName1, final String fileName2) throws Exception {

		try (	FileInputStream fin1 = new FileInputStream(fileName1);
				FileInputStream fin2 = new FileInputStream(fileName2);
				FileChannel fcin1 = fin1.getChannel();
				FileChannel fcin2 = fin2.getChannel();
				) {
			ByteBuffer buffer1 = ByteBuffer.allocate(BUFFERSIZE);
			ByteBuffer buffer2 = ByteBuffer.allocate(BUFFERSIZE);
			int lu1, lu2;
			do {
				buffer1.clear();
				buffer2.clear();
				lu1 = fcin1.read(buffer1);
				lu2 = fcin2.read(buffer2);
				if (lu1 != lu2) {

					return false;
				}
				buffer1.flip();
				buffer2.flip();
				for (int ind=0; ind < lu1; ind++) {
					if (buffer1.get() != buffer2.get()) {

						return false;
					}
				}

			} while (lu1 == BUFFERSIZE);


			return true;
		} // les close des fichiers est automatique avec la syntaxe try () { }
	}


	/**
	 * Remplit un fichier de bytes aléatoires. Le fichier est plus grand que les ByteBuffer utilisés
	 * pour les tests afin d'être sûr que plusieurs tours de boucles soient nécessaires pour le
	 * lire.
	 * @param fileName
	 *            nom du fichier à remplir.
	 * @param randomSeed
	 *            graine du générateur de la série des nombres pseudo-aléatoires mis
	 *            dans le fichier. 
	 */
	private static void createDataTestFile(final String fileName, final int randomSeed, final int dataSize) {
		byte [] randomBytes = new byte[dataSize];
		// générateur de nombres pseudo-aléatoires. Attention pour rendre les
		// tests reproductibles, il faut fixer le début de la suite pour régénérer
		// le même fichier à chaque lancement des tests.
		Random random = new Random(randomSeed);

		// tableau remplis d'octets aléatoires
		random.nextBytes(randomBytes);


		ByteBuffer buffer = ByteBuffer.wrap(randomBytes);
		// après un wrap() le curseur position est déjà à 0. Pas de flip() à faire.

		try (FileOutputStream fout = new FileOutputStream(fileName);
				FileChannel fcout = fout.getChannel();) {

			fcout.write(buffer);

		} catch (FileNotFoundException fnfe) {
			System.out.println(fnfe);
		} catch (IOException ioe) {
			System.out.println(ioe);
		}
		// le close des fichiers est automatique avec la syntaxe try () { }
	}


	@BeforeEach
	public  void avantTests() {
		getThread = null;
	}

	@AfterEach
	public void apresTests() {
		if ((getThread!= null) && (getThread.isAlive())) {
			getThread.stop();
		}
		getThread = null;
	}

}
